ENT.Base            = "base_anim"
ENT.Type            = "anim"
ENT.PrintName       = "Directional Gravity Controller"
ENT.Author          = "Foohy"
ENT.Contact         = "foohy@foohy.net"
ENT.Purpose         = "Angle = angle of gravity"
ENT.Instructions    = "Change the angle of the arrow to alter the angle of gravity."
ENT.Category        = "Directional Gravity"  
ENT.Spawnable       = false
ENT.AdminSpawnable  = true

if SERVER then

    local Affectply = false
    concommand.Add( "arrow_affectplayers", function()
    
        Affectply = !Affectply
        
    end )

    function ENT:SpawnFunction( ply, tr )

        if !tr.Hit then return end

        local SpawnPos = tr.HitPos + tr.HitNormal * 16

        local ent = ents.Create( "directional_gravity" )
        ent:SetPos( SpawnPos + Vector(0,0,27) )
        ent:Spawn()
        ent:Activate()

        for k, v in ipairs( ents.GetAll() ) do

            if IsValid( v ) && v != ent && v:GetClass() == "directional_gravity" then
            
                v:Remove()

            end

        end

        ent:SetName( "Direction Gravity Controller" )

        return ent

    end
    
    function ENT:Initialize()

        self:SetModel( "models/gmod_tower/arrow.mdl" )
        self:PhysicsInit( SOLID_VPHYSICS )
        self:SetMoveType( MOVETYPE_VPHYSICS )
        self:SetSolid( SOLID_VPHYSICS )
        self:SetUseType( SIMPLE_USE ) // Or else it'll go WOBLBLBLBLBLBLBLBL

        self.IsEnabled = true


    end
    
    function ENT:Think()
    
        if !IsValid( self ) || !self.IsEnabled then return end

        local angle = self:GetUp() * - 600

        if !self.LastAngle || self.LastAngle != angle then //let's only change this when needed

            physenv.SetGravity( angle ) //set the gravity of everything to this angle
            self.LastAngle = angle //store this angle
            
            
            
        end

        for k, v in ipairs( ents.GetAll() ) do
        

            local phys = v:GetPhysicsObject()
            if IsValid( phys ) then
                phys:Wake() //we need to constantly wake the objects or they'll be ignored by this property
            end
            
            if IsValid( v ) && v:IsPlayer() && Affectply then
            
                if angle.z < 0 then angle.z = 0 end
                    
                 v:SetVelocity( angle/2 )
        
            end
                  
            

        end

    end

    function ENT:Use( ply )
    
        if !IsValid( ply ) || !ply:IsPlayer() then return end
        
        self:ToggleControl()
		
		self.LastAngle = nil 

    end
    
    function ENT:ToggleControl()
    
        self.IsEnabled = !self.IsEnabled
        
        if !self.IsEnabled then

            self:TurnOff()
        
        end
        
        umsg.Start( "ToggleGravityController" )
            umsg.Entity( self )
            umsg.Bool( self.IsEnabled )
        umsg.End()

    end    

    
    
    
    function ENT:TurnOff()

        physenv.SetGravity( Vector( 0, 0, -600 ) )

        //wake everything to reset (just incase)
        for k, v in ipairs( ents.GetAll() ) do

            local phys = v:GetPhysicsObject()
            if IsValid( phys ) then
                phys:Wake()
            end

        end
        
    end

    function ENT:OnRemove()

        self:TurnOff()

    end

else //CLIENT

    ENT.RenderGroup = RENDERGROUP_OPAQUE
    ENT.Sprite = Material( "effects/blueflare1" )
    
    function ENT:Draw()

        self:DrawModel()
    
        if !self.IsEnabled then return end

        render.SetMaterial( self.Sprite )
        render.DrawSprite( self:GetPos(), 50, 50, Color( 255, 0, 0, 255 ) )
        
    end

    function ENT:Initialize()

        self.IsEnabled = true
        self:CreateEmitter()

    end
    
    function ENT:Think()
    
        if self.IsEnabled then

            self:CreateEmitter()
            
        else
        
            self:RemoveEmitter()
            return
            
        end
        
        if !self.NextParticle || CurTime() > self.NextParticle then

            local pos = self:GetPos() + ( VectorRand() * ( self:BoundingRadius() * 0.75 ) )
            local vel = VectorRand() * 3

            vel.z = vel.z * ( vel.z > 0 && -3 or 3 )

            local particle = self.Emitter:Add( "sprites/powerup_effects", pos )

            if particle then
                particle:SetVelocity( vel )
                particle:SetDieTime( math.Rand( 1, 3 ) )
                particle:SetStartAlpha( 255 )
                particle:SetEndAlpha( 0 )
                particle:SetStartSize( 10 )
                particle:SetEndSize( 0 )
                particle:SetRoll( math.Rand( 0, 360 ) )
                particle:SetRollDelta( math.Rand( -5.5, 5.5 ) )
                particle:SetColor( math.random( 50, 255 ), 0, 0 )
            end

            self.NextParticle = CurTime() + 0.15

        end
        
        local dlight = DynamicLight( self:EntIndex() )
        if dlight then

            dlight.Pos = self:GetPos()
            dlight.r = 255
            dlight.g = 0
            dlight.b = 0
            dlight.Brightness = 0.35
            dlight.Size = 128
            dlight.Decay = 128 * 2
            dlight.DieTime = CurTime() + 0.5

        end

    end
    
    function ENT:CreateEmitter()

        if self.Emitter then return end //we already have one!

        self.Emitter = ParticleEmitter( self:GetPos() )

    end
    
    function ENT:RemoveEmitter()
        
        if !self.Emitter then return end //we don't have an emitter!

        self.Emitter:Finish()
        self.Emitter = nil

    end

    function ENT:OnRemove()

        self:RemoveEmitter()

    end

    usermessage.Hook( "ToggleGravityController", function( um )

        local ent = um:ReadEntity()
        if !ValidEntity( ent ) then return end

        ent.IsEnabled = um:ReadBool()

    end    )

end